==> dabstick-radio-0.92.tgz: dabstick-radio-0.92/dabreceiver-V2/backend/deconvolve.cpp <==
#
/*
 * 	The deconvolution for both uep and eep
 */
#include	"deconvolve.h"
#include	"protTables.h"

struct protectionProfile {
	int16_t	bitRate;
	int16_t	protLevel;
	int16_t L1;
	int16_t	L2;
	int16_t	L3;
	int16_t	L4;
	int16_t	PI1;
	int16_t	PI2;
	int16_t	PI3;
	int16_t	PI4;
} profileTable [] = {
	{32,	5,	3, 4, 27, 0,	5, 3, 2, -1},
	{32,	4,	3, 3, 18, 0,	11, 6, 5, -1},
	{32,	3,	3, 4, 14, 3,	15, 9, 6, 8},
	{32,	2,	3, 4, 14, 3,	22, 13, 8, 13},
	{32,	1,	3, 5, 13, 3,	24, 17, 12, 17},

	{48,	5,	4, 3, 26, 3,	5, 4, 2, 3},
	{48,	4,	3, 4, 26, 3,	9, 6, 4, 6},
	{48,	3,	3, 4, 26, 3,	15, 10, 6, 9},
	{48,	2,	3, 4, 26, 3,	24, 14, 8, 15},
	{48,	1,	3, 5, 25, 3,	24, 18, 13, 18},

	{64,	5,	6, 9, 31, 2,	5, 3, 2, 3},
	{64,	4,	6, 9, 33, 0,	11, 6, 6, -1},
	{64,	3,	6, 12, 27, 3,	16, 8, 6, 9},
	{64,	2,	6, 10, 29, 3,	23, 13, 8, 13},
	{64,	1,	6, 11, 28, 3,	24, 18, 12, 18},

	{96,	5,	7, 9, 53, 3,	5, 4, 2, 4},
	{96,	4,	7, 10, 52, 3,	9, 6, 4, 6},
	{96,	3,	6, 12, 51, 3,	16, 9, 6, 10},
	{96,	2,	6, 10, 53, 3,	22, 12, 9, 12},
	{96,	1,	6, 13, 50, 3,	24, 18, 13, 19},

	{128,	5,	12, 19, 62, 3,	5, 3, 2, 4},
	{128,	4,	11, 21, 61, 3,	11, 6, 5, 7},
	{128,	3,	11, 22, 60, 3,	16, 9, 6, 10},
	{128,	2,	11, 21, 61, 3,	22, 12, 9, 14},
	{128,	1,	11, 20, 62, 3,	24, 17, 13, 19},

	{160,	5,	11, 19, 87, 3,	5, 4, 2, 4},
	{160,	4,	11, 23, 83, 3,	11, 6, 5, 9},
	{160,	3,	11, 24, 82, 3,	16, 8, 6, 11},
	{160,	2,	11, 21, 85, 3,	22, 11, 9, 13},
	{160,	1,	11, 22, 84, 3,	24, 18, 12, 19},

	{192,	5,	11, 20, 110, 3,	6, 4, 2, 5},
	{192,	4,	11, 22, 108, 3,	10, 6, 4, 9},
	{192,	3,	11, 24, 106, 3, 16, 10, 6, 11},
	{192,	2,	11, 20, 110, 3, 22, 13, 9, 13},
	{192,	1,	11, 21, 109, 3,	24, 20, 13, 24},

	{224,	5,	12, 22, 131, 3,	8,  6, 2, 6},
	{224,	4,	12, 26, 127, 3,	12, 8, 4, 11},
	{224,	3,	11, 20, 134, 3, 16, 10, 7, 9},
	{224,	2,	11, 22, 132, 3,	24, 16, 10, 15},
	{224,	1,	11, 24, 130, 3,	24, 20, 12, 20},

	{0,	-1,	-1, -1, -1, -1,	-1, -1, -1, -1}
};

static
int16_t	findIndex (int16_t bitRate, int16_t protLevel) {
int16_t	i;

	for (i = 0; profileTable [i].bitRate != 0; i ++)
	   if ((profileTable [i]. bitRate == bitRate) &&
	       (profileTable [i]. protLevel == protLevel))
	      return i;

	return -1;
}

	uep_deconvolve::uep_deconvolve (int16_t bitRate,
	                                int16_t protLevel,
	                                int16_t tracebackLevel):viterbi (31) {
int16_t	index;

	this	-> bitRate	= bitRate;
	this	-> tracebackLevel	= tracebackLevel;
	index	= findIndex (bitRate, protLevel);
	if (index == -1) {
	   fprintf (stderr, "%d (%d) has a problem\n", bitRate, protLevel);
	   index = 1;
	}
	L1	= profileTable [index]. L1;
	L2	= profileTable [index]. L2;
	L3	= profileTable [index]. L3;
	L4	= profileTable [index]. L4;

	PI1	= get_PCodes (profileTable [index]. PI1 -1);
	PI2	= get_PCodes (profileTable [index]. PI2 -1);
	PI3	= get_PCodes (profileTable [index]. PI3 -1);
	if ((profileTable [index]. PI4 - 1) != -1)
	   PI4	= get_PCodes (profileTable [index]. PI4 -1);
	else
	   PI4	= NULL;
}

	uep_deconvolve::~uep_deconvolve (void) {
}

bool	uep_deconvolve::deconvolve (int16_t *v,
	                            int32_t size, uint8_t *outBuffer) {
int16_t	i, j;
int32_t	outSize, costs;
int16_t	res;
int16_t	bit;
int16_t	inputCounter	= 0;
int16_t	outputCounter	= 0;

	(void)size;			// currently unused
	outSize	= 24 * bitRate;
	viterbi_reset (8 * tracebackLevel - 1, 8);
//
//	according to the standard we process the logical frame
//	with a pair of tuples
//	(L1, PI1), (L2, PI2), (L3, PI3), (L4, PI4)
//
	for (i = 0; i < L1; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI1 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	for (i = 0; i < L2; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI2 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	for (i = 0; i < L3; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI3 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	for (i = 0; i < L4; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI4 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

//	fprintf (stderr, "Inputcounter = %d, inputsize = %d\n",
//		          inputCounter, size);
//
//	Dumping the bits from the traceback buffer
	for (i = 0; i < v_traceLength; i ++) {
	   for (j = 0; j < 4; j ++) {
	      res = viterbi_shiftbit (0, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	if  (outputCounter != outSize) 
	fprintf (stderr, "exit 1: we had %d bits and fill to %d bits\n",
	                 outputCounter, outSize);
	for (; outputCounter < outSize; outputCounter ++)
	   outBuffer [outputCounter] = 0;
	return true;
}
//
////
	eep_deconvolve::eep_deconvolve (int16_t bitRate,
	                                int16_t protLevel,
	                                int16_t tracebackLevel):viterbi (31) {
	this	-> bitRate = bitRate;
	this	-> tracebackLevel	= tracebackLevel;
	if (protLevel & 0100) {	// set A profiles
	   switch (protLevel & 07) {
	      case 1:
	         L1	= 6 * bitRate / 8 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (24 - 1);
	         PI2	= get_PCodes (23 - 1);
	         break;

	      case 2:
	         if (bitRate == 8) {
	            L1	= 5;
	            L2	= 1;
	            PI1	= get_PCodes (13 - 1);
	            PI2	= get_PCodes (12 - 1);
	         } else {
	            L1	= 2 * bitRate / 8 - 3;
	            L2	= 4 * bitRate / 8 + 3;
	            PI1	= get_PCodes (14 - 1);
	            PI2	= get_PCodes (13 - 1);
	         }
	         break;

	      case 3:
	         L1	= 6 * bitRate / 8 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (8 - 1);
	         PI2	= get_PCodes (7 - 1);
	         break;

	      case 4:
	         L1	= 4 * bitRate / 8 - 3;
	         L2	= 2 * bitRate / 8 + 3;
	         PI1	= get_PCodes (3 - 1);
	         PI2	= get_PCodes (2 - 1);
	         break;
	   }
	}
	else
	if (protLevel & 0200) {		// B series
	   switch ((protLevel & 07)) {
	      case 4:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (2 - 1);
	         PI2	= get_PCodes (1 - 1);
	         break;

	      case 3:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (4 - 1);
	         PI2	= get_PCodes (3 - 1);
	         break;

	      case 2:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (6 - 1);
	         PI2	= get_PCodes (5 - 1);
	         break;

	      case 1:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (10 - 1);
	         PI2	= get_PCodes (9 - 1);
	         break;
	   }
	}
}

	eep_deconvolve::~eep_deconvolve (void) {
}

bool	eep_deconvolve::deconvolve (int16_t *v,
	                            int32_t size, uint8_t *outBuffer) {

int32_t	outSize;
int16_t	bit;
int16_t	i, j;
int16_t	res;
int32_t	costs;
int32_t	inputCounter	= 0;
int32_t	outputCounter	= 0;

	(void)size;			// currently unused
	outSize	= 24 * bitRate;
	viterbi_reset (8 * tracebackLevel - 1, 8);
//
//	according to the standard we process the logical frame
//	with a pair of tuples
//	(L1, PI1), (L2, PI2), (L3, PI3), (L4, PI4)
//
	for (i = 0; i < L1; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI1 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	for (i = 0; i < L2; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI2 [j % 32] == 1)
	         bit = v [inputCounter ++];
	      else
	         bit = 0;	
	      res = viterbi_shiftbit (bit, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}
//	Dumping the bits from the traceback buffer
	for (i = 0; i < v_traceLength; i ++) {
	   for (j = 0; j < 4; j ++) {
	      res = viterbi_shiftbit (0, &costs);
	      if (res >= 0) { 
	         outBuffer [outputCounter ++] = (res & 0200) >> 7;
	         outBuffer [outputCounter ++] = (res & 0100) >> 6;
	         outBuffer [outputCounter ++] = (res & 0040) >> 5;
	         outBuffer [outputCounter ++] = (res & 0020) >> 4;
	         outBuffer [outputCounter ++] = (res & 0010) >> 3;
	         outBuffer [outputCounter ++] = (res & 0004) >> 2;
	         outBuffer [outputCounter ++] = (res & 0002) >> 1;
	         outBuffer [outputCounter ++] = (res & 0001);
	      }
	   }
	}

	if  (outputCounter != outSize) 
	   fprintf (stderr, "exit 2: we had %d bits and fill to %d bits\n",
	                 outputCounter, outSize);
	for (; outputCounter < outSize; outputCounter ++)
	   outBuffer [outputCounter] = 0;
	return true;
}

==> dabstick-radio-0.95.tgz: dabstick-radio-0.95/dabreceiver-V2/backend/deconvolve.cpp <==
#
/*
 * 	The deconvolution for both uep and eep
 */
#include	"deconvolve.h"
#include	"protTables.h"
#include	<stdio.h>
#include	"jff-include.h"

struct protectionProfile {
	int16_t	bitRate;
	int16_t	protLevel;
	int16_t L1;
	int16_t	L2;
	int16_t	L3;
	int16_t	L4;
	int16_t	PI1;
	int16_t	PI2;
	int16_t	PI3;
	int16_t	PI4;
} profileTable [] = {
	{32,	5,	3, 4, 27, 0,	5, 3, 2, -1},
	{32,	4,	3, 3, 18, 0,	11, 6, 5, -1},
	{32,	3,	3, 4, 14, 3,	15, 9, 6, 8},
	{32,	2,	3, 4, 14, 3,	22, 13, 8, 13},
	{32,	1,	3, 5, 13, 3,	24, 17, 12, 17},

	{48,	5,	4, 3, 26, 3,	5, 4, 2, 3},
	{48,	4,	3, 4, 26, 3,	9, 6, 4, 6},
	{48,	3,	3, 4, 26, 3,	15, 10, 6, 9},
	{48,	2,	3, 4, 26, 3,	24, 14, 8, 15},
	{48,	1,	3, 5, 25, 3,	24, 18, 13, 18},

	{64,	5,	6, 9, 31, 2,	5, 3, 2, 3},
	{64,	4,	6, 9, 33, 0,	11, 6, 6, -1},
	{64,	3,	6, 12, 27, 3,	16, 8, 6, 9},
	{64,	2,	6, 10, 29, 3,	23, 13, 8, 13},
	{64,	1,	6, 11, 28, 3,	24, 18, 12, 18},

	{96,	5,	7, 9, 53, 3,	5, 4, 2, 4},
	{96,	4,	7, 10, 52, 3,	9, 6, 4, 6},
	{96,	3,	6, 12, 51, 3,	16, 9, 6, 10},
	{96,	2,	6, 10, 53, 3,	22, 12, 9, 12},
	{96,	1,	6, 13, 50, 3,	24, 18, 13, 19},

	{128,	5,	12, 19, 62, 3,	5, 3, 2, 4},
	{128,	4,	11, 21, 61, 3,	11, 6, 5, 7},
	{128,	3,	11, 22, 60, 3,	16, 9, 6, 10},
	{128,	2,	11, 21, 61, 3,	22, 12, 9, 14},
	{128,	1,	11, 20, 62, 3,	24, 17, 13, 19},

	{160,	5,	11, 19, 87, 3,	5, 4, 2, 4},
	{160,	4,	11, 23, 83, 3,	11, 6, 5, 9},
	{160,	3,	11, 24, 82, 3,	16, 8, 6, 11},
	{160,	2,	11, 21, 85, 3,	22, 11, 9, 13},
	{160,	1,	11, 22, 84, 3,	24, 18, 12, 19},

	{192,	5,	11, 20, 110, 3,	6, 4, 2, 5},
	{192,	4,	11, 22, 108, 3,	10, 6, 4, 9},
	{192,	3,	11, 24, 106, 3, 16, 10, 6, 11},
	{192,	2,	11, 20, 110, 3, 22, 13, 9, 13},
	{192,	1,	11, 21, 109, 3,	24, 20, 13, 24},

	{224,	5,	12, 22, 131, 3,	8,  6, 2, 6},
	{224,	4,	12, 26, 127, 3,	12, 8, 4, 11},
	{224,	3,	11, 20, 134, 3, 16, 10, 7, 9},
	{224,	2,	11, 22, 132, 3,	24, 16, 10, 15},
	{224,	1,	11, 24, 130, 3,	24, 20, 12, 20},

	{0,	-1,	-1, -1, -1, -1,	-1, -1, -1, -1}
};

static
uint8_t	PI_X [24] = {
	1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0,
	1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0
};

static
int16_t	findIndex (int16_t bitRate, int16_t protLevel) {
int16_t	i;

	for (i = 0; profileTable [i].bitRate != 0; i ++)
	   if ((profileTable [i]. bitRate == bitRate) &&
	       (profileTable [i]. protLevel == protLevel))
	      return i;

	return -1;
}

	uep_deconvolve::uep_deconvolve (int16_t bitRate,
	                                int16_t protLevel):viterbi (24 * bitRate) {
int16_t	index;

	this	-> bitRate		= bitRate;
	index	= findIndex (bitRate, protLevel);
	if (index == -1) {
	   fprintf (stderr, "%d (%d) has a problem\n", bitRate, protLevel);
	   index = 1;
	}
	outSize		= 24 * bitRate;
	viterbiBlock	= new int16_t [outSize * 4 + 24];
	L1	= profileTable [index]. L1;
	L2	= profileTable [index]. L2;
	L3	= profileTable [index]. L3;
	L4	= profileTable [index]. L4;

	PI1	= get_PCodes (profileTable [index]. PI1 -1);
	PI2	= get_PCodes (profileTable [index]. PI2 -1);
	PI3	= get_PCodes (profileTable [index]. PI3 -1);
	if ((profileTable [index]. PI4 - 1) != -1)
	   PI4	= get_PCodes (profileTable [index]. PI4 -1);
	else
	   PI4	= NULL;
}

	uep_deconvolve::~uep_deconvolve (void) {
	delete[]	viterbiBlock;
}

bool	uep_deconvolve::deconvolve (int16_t *v,
	                            int32_t size, uint8_t *outBuffer) {
int16_t	i, j;
int16_t	inputCounter	= 0;
int32_t	viterbiCounter	= 0;
uint8_t	tempBuffer	[10000];
	(void)size;			// currently unused

//	according to the standard we process the logical frame
//	with a pair of tuples
//	(L1, PI1), (L2, PI2), (L3, PI3), (L4, PI4)
//
	for (i = 0; i < L1; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI1 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter ++]  = 0;	
	   }
	}

	for (i = 0; i < L2; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI2 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter++] = 0;	
	   }
	}

	for (i = 0; i < L3; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI3 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter ++] = 0;	
	   }
	}

	for (i = 0; i < L4; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI4 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter ++] = 0;	
	   }
	}

//	we had a final block of 24 bits  with puncturing according to PI_X
//	This block constitues the 6 * 4 bits of the register itself.
	for (i = 0; i < 24; i ++) {
	   if (PI_X [i] == 1)  
	      viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	   else
	      viterbiBlock [viterbiCounter ++] = 0;
	}
	viterbi::deconvolve (viterbiBlock, outBuffer);
	return true;
}

//
//
	eep_deconvolve::eep_deconvolve (int16_t bitRate,
	                                int16_t protLevel):viterbi (24 * bitRate) {
	this	-> bitRate = bitRate;
	outSize		= 24 * bitRate;
	viterbiBlock	= new int16_t [outSize * 4 + 24];
	if (protLevel & 0100) {	// set A profiles
	   switch (protLevel & 07) {
	      case 1:
	         L1	= 6 * bitRate / 8 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (24 - 1);
	         PI2	= get_PCodes (23 - 1);
	         break;

	      case 2:
	         if (bitRate == 8) {
	            L1	= 5;
	            L2	= 1;
	            PI1	= get_PCodes (13 - 1);
	            PI2	= get_PCodes (12 - 1);
	         } else {
	            L1	= 2 * bitRate / 8 - 3;
	            L2	= 4 * bitRate / 8 + 3;
	            PI1	= get_PCodes (14 - 1);
	            PI2	= get_PCodes (13 - 1);
	         }
	         break;

	      case 3:
	         L1	= 6 * bitRate / 8 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (8 - 1);
	         PI2	= get_PCodes (7 - 1);
	         break;

	      case 4:
	         L1	= 4 * bitRate / 8 - 3;
	         L2	= 2 * bitRate / 8 + 3;
	         PI1	= get_PCodes (3 - 1);
	         PI2	= get_PCodes (2 - 1);
	         break;
	   }
	}
	else
	if (protLevel & 0200) {		// B series
	   switch ((protLevel & 07)) {
	      case 4:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (2 - 1);
	         PI2	= get_PCodes (1 - 1);
	         break;

	      case 3:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (4 - 1);
	         PI2	= get_PCodes (3 - 1);
	         break;

	      case 2:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (6 - 1);
	         PI2	= get_PCodes (5 - 1);
	         break;

	      case 1:
	         L1	= 24 * bitRate / 32 - 3;
	         L2	= 3;
	         PI1	= get_PCodes (10 - 1);
	         PI2	= get_PCodes (9 - 1);
	         break;
	   }
	}
}

	eep_deconvolve::~eep_deconvolve (void) {
	delete[]	viterbiBlock;
}

bool	eep_deconvolve::deconvolve (int16_t *v,
	                            int32_t size, uint8_t *outBuffer) {

int16_t	i, j;
int32_t	inputCounter	= 0;
int32_t	viterbiCounter	= 0;
	(void)size;			// currently unused
//
//	according to the standard we process the logical frame
//	with a pair of tuples
//	(L1, PI1), (L2, PI2), (L3, PI3), (L4, PI4)
//
	for (i = 0; i < L1; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI1 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter ++] = 0;	
	   }
	}

	for (i = 0; i < L2; i ++) {
	   for (j = 0; j < 128; j ++) {
	      if (PI2 [j % 32] == 1)
	         viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	      else
	         viterbiBlock [viterbiCounter ++] = 0;	
	   }
	}
//	we had a final block of 24 bits  with puncturing according to PI_X
//	This block constitues the 6 * 4 bits of the register itself.
	for (i = 0; i < 24; i ++) {
	   if (PI_X [i] == 1)  
	      viterbiBlock [viterbiCounter ++] = v [inputCounter ++];
	   else
	      viterbiBlock [viterbiCounter ++] = 0;
	}

	viterbi::deconvolve (viterbiBlock, outBuffer);
	return true;
}
